# Seen - track when a person speaks, and allow data to be queried
# Copyright (C) 2010  Brian S. Stephan
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ConfigParser import NoOptionError
import re

from irclib import irclib
from datetime import datetime
from dateutil.tz import *

from Module import Module

# Keeps track of when people say things in public channels, and reports on when
# they last said something.

class Seen(Module):

    def register_handlers(self, server):
        server.add_global_handler('pubmsg', self.on_pubmsg)
        server.add_global_handler('privmsg', self.on_privmsg)

    def unregister_handlers(self):
        self.server.remove_global_handler('pubmsg', self.on_pubmsg)
        self.server.remove_global_handler('privmsg', self.on_privmsg)

    # Overriding the default because we need stuff to occur before the addressed
    # and so on checks.

    def on_pubmsg(self, connection, event):
        nick = irclib.nm_to_n(event.source())
        userhost = irclib.nm_to_uh(event.source())
        replypath = event.target()
        what = event.arguments()[0]

        admin_unlocked = False

        # whatever it is, store it
        if not self.config.has_section('seen'):
            self.config.add_section('seen')

        self.config.set('seen', nick, userhost + '|:|' + datetime.utcnow().isoformat() + '|:|' + what)

        try:
            if userhost == self.config.get('admin', 'userhost'):
                admin_unlocked = True
        except NoOptionError: pass

        # only do commands if the bot has been addressed directly
        addressed_pattern = '^' + connection.get_nickname() + '[:,]?\s+'
        addressed_re = re.compile(addressed_pattern)

        if not addressed_re.match(what):
            return
        else:
            what = addressed_re.sub('', what)

        if replypath is not None:
            what = self.try_recursion(connection, event, nick, userhost, replypath, what, admin_unlocked)

        # try reloading
        self.reload(connection, event, nick, userhost, replypath, what, admin_unlocked)

        self.do(connection, event, nick, userhost, replypath, what, admin_unlocked)

    def do(self, connection, event, nick, userhost, replypath, what, admin_unlocked):
        whats = what.split(' ')
        if whats[0] == 'seen' and len(whats) >= 2:
            query = whats[1]
            if query != 'debug':
                try:
                    seendata = self.config.get('seen', query).split('|:|')
                    converted = datetime.strptime(seendata[1], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=tzutc())
                    replystr = 'last saw ' + query + ' at ' + converted.astimezone(tzlocal()).strftime("%Y/%m/%d %H:%M:%S %Z") + ' saying \'' + seendata[2] + '\''
                    return self.reply(connection, replypath, replystr)
                except NoOptionError: pass

# vi:tabstop=4:expandtab:autoindent
# kate: indent-mode python;indent-width 4;replace-tabs on;
